@@ -10,6 +10,7 @@ from types import SimpleNamespace
 from typing import Optional
 
 import click
+import jinja2
 import pkg_resources
 import pygments
 import yaml
@@ -24,7 +25,7 @@ from termcolor import colored
 
 from typhoon import connections
 from typhoon.cli_helpers.cli_completion import get_remote_names, get_dag_names, get_conn_envs, get_conn_ids, \
-    get_var_types, get_node_names, get_edge_names, get_deploy_targets
+    get_var_types, get_node_names, get_edge_names, get_deploy_targets, PROJECT_TEMPLATES
 from typhoon.cli_helpers.status import dags_with_changes, dags_without_deploy, check_connections_yaml, \
     check_connections_dags, check_variables_dags
 from typhoon.connections import Connection
@@ -35,6 +36,8 @@ from typhoon.core.glue import get_dag_errors, load_dag, load_dag_definition
 from typhoon.core.settings import Settings
 from typhoon.deployment.packaging import build_all_dags
 from typhoon.handler import run_dag
+from typhoon.introspection.introspect_extensions import get_typhoon_extensions, get_typhoon_extensions_info, \
+    get_hooks_info
 from typhoon.introspection.introspect_transformations import run_transformations, TransformationResult
 from typhoon.local_config import EXAMPLE_CONFIG
 from typhoon.metadata_store_impl.sqlite_metadata_store import SQLiteMetadataStore
@@ -74,29 +77,39 @@ def cli():
 @click.argument('project_name')
 @click.option('--deploy-target', autocompletion=get_deploy_targets, required=False, default='typhoon',
               help='Target for DAG deployment. It can be "typhoon" or "airflow"')
-@click.option('--template', required=False, default='hello_world')
+@click.option('--template', required=False, default='hello_world',
+              help=f'Project template. One of {PROJECT_TEMPLATES}')
 def init(project_name: str, deploy_target: str, template: str):
     """Create a new Typhoon project"""
     example_project_path = Path(pkg_resources.resource_filename('typhoon', 'examples')) / template
     dest = Path.cwd() / project_name
     shutil.copytree(str(example_project_path), str(dest))
 
-    if template == 'airflow_docker':
-        cfg_path = (dest / 'src/typhoon.cfg')
-        dag_schema_path = (dest / 'src/dag_schema.json')
-        component_schema_path = (dest / 'src/component_schema.json')
+    if template == 'extension':
+        module_path = dest / 'module'
+        module_path.rename(dest / project_name)
+        setup_path = dest / 'setup.py'
+        setup_path.write_text(
+            data=jinja2.Template(setup_path.read_text()).render(name=project_name))
+        print(f'Extension created in {dest}')
     else:
-        cfg_path = (dest / 'typhoon.cfg')
-        dag_schema_path = (dest / 'dag_schema.json')
-        component_schema_path = (dest / 'component_schema.json')
+        if template == 'airflow_docker':
+            cfg_path = dest / 'src/typhoon.cfg'
+            dag_schema_path = dest / 'src/dag_schema.json'
+            component_schema_path = dest / 'src/component_schema.json'
+        else:
+            cfg_path = (dest / 'typhoon.cfg')
+            dag_schema_path = (dest / 'dag_schema.json')
+            component_schema_path = (dest / 'component_schema.json')
 
-    cfg_path.write_text(EXAMPLE_CONFIG.format(project_name=project_name, deploy_target=deploy_target))
-    dag_schema_path.write_text(DAGDefinitionV2.schema_json(indent=2))
-    component_schema_path.write_text(Component.schema_json(indent=2))
-    print(f'Project created in {dest}')
-    print('If you want auto completion run the following:')
-    for shell_type in ['bash', 'zsh', 'fish']:
-        print(f'In {shell_type}', colored(f'eval "$(_TYPHOON_COMPLETE=source_{shell_type} typhoon)"', 'blue'))
+        cfg_path.write_text(EXAMPLE_CONFIG.format(project_name=project_name, deploy_target=deploy_target))
+        dag_schema_path.write_text(DAGDefinitionV2.schema_json(indent=2))
+        component_schema_path.write_text(Component.schema_json(indent=2))
+
+        print(f'Project created in {dest}')
+        print('If you want auto completion run the following:')
+        for shell_type in ['bash', 'zsh', 'fish']:
+            print(f'In {shell_type}', colored(f'eval "$(_TYPHOON_COMPLETE=source_{shell_type} typhoon)"', 'blue'))
 
 
 @cli.command()
@@ -256,7 +269,7 @@ def list_dags(remote: Optional[str], long: bool):
 
 
 @cli_dags.command(name='build')
-@click.argument('dag_name', autocompletion=get_dag_names, required=False, default=None)
+@click.option('--dag-name', autocompletion=get_dag_names, required=False, default=None)
 @click.option('--all', 'all_', is_flag=True, default=False, help='Build all DAGs (mutually exclusive with DAG_NAME)')
 def build_dags(dag_name: Optional[str], all_: bool):
     """Build code for dags in $TYPHOON_HOME/out/"""
@@ -823,6 +836,72 @@ def remove_variable(remote: Optional[str], var_id: str):
     print(f'Variable {var_id} deleted')
 
 
+@cli.group(name='extension')
+def cli_extension():
+    """Manage Typhoon extensions"""
+    pass
+
+
+@cli_extension.command(name='ls')
+@click.argument('remote', autocompletion=get_remote_names, required=False, default=None)
+def list_extensions(remote: Optional[str]):
+    # set_settings_from_remote(remote)
+    for extension_info, _ in get_typhoon_extensions():
+        print(extension_info.name)
+
+
+@cli_extension.group(name='functions')
+def cli_extension_functions():
+    """Manage Typhoon extension functions"""
+    pass
+
+
+@cli_extension_functions.command(name='ls')
+@click.argument('remote', autocompletion=get_remote_names, required=False, default=None)
+def list_extension_functions(remote: Optional[str]):
+    # set_settings_from_remote(remote)
+    header = ['Module', 'Path']
+    table_body = []
+    for module_name, module_path in get_typhoon_extensions_info()['functions'].items():
+        table_body.append([module_name, module_path])
+    print(tabulate(table_body, header, 'plain'))
+
+
+@cli_extension.group(name='transformations')
+def cli_extension_transformations():
+    """Manage Typhoon extension transformations"""
+    pass
+
+
+@cli_extension_transformations.command(name='ls')
+@click.argument('remote', autocompletion=get_remote_names, required=False, default=None)
+def list_extension_transformations(remote: Optional[str]):
+    # set_settings_from_remote(remote)
+    header = ['Module', 'Path']
+    table_body = []
+    for module_name, module_path in get_typhoon_extensions_info()['transformations'].items():
+        table_body.append([module_name, module_path])
+    print(tabulate(table_body, header, 'plain'))
+
+
+@cli_extension.group(name='hooks')
+def cli_extension_hooks():
+    """Manage Typhoon extension transformations"""
+    pass
+
+
+@cli_extension_hooks.command(name='ls')
+@click.argument('remote', autocompletion=get_remote_names, required=False, default=None)
+def list_extension_hooks(remote: Optional[str]):
+    # set_settings_from_remote(remote)
+    header = ['Hook Type', 'Class name']
+    table_body = []
+    extensions_info = get_typhoon_extensions_info()
+    for conn_type, cls in get_hooks_info(extensions_info).items():
+        table_body.append([conn_type, cls.__name__])
+    print(tabulate(table_body, header, 'plain'))
+
+
 class SubprocessError(Exception):
     pass
 
